@@ -6,8 +6,8 @@ demo_dir = os.path.dirname(os.path.realpath(__file__))
 
 ufl_files = []
 for file in os.listdir(demo_dir):
-    if file.endswith(".ufl") and not file.startswith("."):
-        ufl_files.append(file[:-4])
+    if file.endswith(".py") and not file == "test_demos.py":
+        ufl_files.append(file[:-3])
 
 
 @pytest.mark.parametrize("file", ufl_files)
@@ -19,12 +19,16 @@ def test_demo(file):
         "Mini",  # EnrichedElement
         "MixedGradient", "TraceElement",  # HDiv Trace
         "MassHdiv_2D_1", "MassHdiv_2D_3", "MixedPoisson", "MassHdiv_2D_2",  # Brezzi-Douglas-Marini
+        "MixedElasticity",  # VectorElement of BDM
+        "RestrictedElement",
+        "_TensorProductElement"
     ]:
         # Skip demos that use elements not yet implemented in basix
         pytest.skip()
 
-    assert os.system(f"cd {demo_dir} && ffcx {file}.ufl") == 0
+    extra_flags = "-Wunused-variable -Werror -fPIC "
+    assert os.system(f"cd {demo_dir} && ffcx {file}.py") == 0
     assert os.system(f"cd {demo_dir} && "
                      "CPATH=../ffcx/codegeneration/ "
-                     f"gcc -I/usr/include/python{sys.version_info.major}.{sys.version_info.minor} -fPIC "
+                     f"gcc -I/usr/include/python{sys.version_info.major}.{sys.version_info.minor} {extra_flags}"
                      f"-shared {file}.c -o {file}.so") == 0
