@@ -3,15 +3,21 @@
 # This file is part of FFCx.(https://www.fenicsproject.org)
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
+
 import collections
-from itertools import product
 import logging
+from typing import Dict, Set, Any, DefaultDict
+from itertools import product
 
 import ufl
+
+from ffcx.codegeneration import geometry
 from ffcx.codegeneration import expressions_template
 from ffcx.codegeneration.backend import FFCXBackend
 from ffcx.codegeneration.C.format_lines import format_indented_lines
+from ffcx.codegeneration.C.cnodes import CNode
 from ffcx.ir.representation import ir_expression
+from ffcx.naming import cdtype_to_numpy
 
 logger = logging.getLogger("ffcx")
 
@@ -25,14 +31,17 @@ def generator(ir, parameters):
     factory_name = ir.name
 
     # Format declaration
-    declaration = expressions_template.declaration.format(factory_name=factory_name)
+    declaration = expressions_template.declaration.format(
+        factory_name=factory_name, name_from_uflfile=ir.name_from_uflfile)
 
     backend = FFCXBackend(ir, parameters)
     L = backend.language
     eg = ExpressionGenerator(ir, backend)
 
     d = {}
+    d["name_from_uflfile"] = ir.name_from_uflfile
     d["factory_name"] = ir.name
+
     parts = eg.generate()
 
     body = format_indented_lines(parts.cs_format(), 1)
@@ -61,9 +70,56 @@ def generator(ir, parameters):
 
     d["num_components"] = len(ir.expression_shape)
     d["num_coefficients"] = len(ir.coefficient_numbering)
+    d["num_constants"] = len(ir.constant_names)
     d["num_points"] = ir.points.shape[0]
     d["topological_dimension"] = ir.points.shape[1]
-    d["needs_facet_permutations"] = 1 if ir.needs_facet_permutations else 0
+    d["scalar_type"] = parameters["scalar_type"]
+    d["np_scalar_type"] = cdtype_to_numpy(parameters["scalar_type"])
+    d["rank"] = len(ir.tensor_shape)
+
+    if len(ir.coefficient_names) > 0:
+        d["coefficient_names_init"] = L.ArrayDecl(
+            "static const char*", f"coefficient_names_{ir.name}", values=ir.coefficient_names,
+            sizes=len(ir.coefficient_names))
+        d["coefficient_names"] = f"coefficient_names_{ir.name}"
+    else:
+        d["coefficient_names_init"] = ""
+        d["coefficient_names"] = L.Null()
+
+    if len(ir.constant_names) > 0:
+        d["constant_names_init"] = L.ArrayDecl(
+            "static const char*", f"constant_names_{ir.name}", values=ir.constant_names,
+            sizes=len(ir.constant_names))
+        d["constant_names"] = f"constant_names_{ir.name}"
+    else:
+        d["constant_names_init"] = ""
+        d["constant_names"] = L.Null()
+
+    code = []
+
+    # FIXME: Should be handled differently, revise how
+    # ufcx_function_space is generated (also for ufcx_form)
+    for (name, (element, dofmap, cmap_family, cmap_degree)) in ir.function_spaces.items():
+        code += [f"static ufcx_function_space function_space_{name}_{ir.name_from_uflfile} ="]
+        code += ["{"]
+        code += [f".finite_element = &{element},"]
+        code += [f".dofmap = &{dofmap},"]
+        code += [f".geometry_family = \"{cmap_family}\","]
+        code += [f".geometry_degree = {cmap_degree}"]
+        code += ["};"]
+
+    d["function_spaces_alloc"] = L.StatementList(code)
+    d["function_spaces"] = ""
+
+    if len(ir.function_spaces) > 0:
+        d["function_spaces"] = f"function_spaces_{ir.name}"
+        d["function_spaces_init"] = L.ArrayDecl("ufcx_function_space*", f"function_spaces_{ir.name}", values=[
+                                                L.AddressOf(L.Symbol(f"function_space_{name}_{ir.name_from_uflfile}"))
+                                                for (name, _) in ir.function_spaces.items()],
+                                                sizes=len(ir.function_spaces))
+    else:
+        d["function_spaces"] = L.Null()
+        d["function_spaces_init"] = ""
 
     # Check that no keys are redundant or have been missed
     from string import Formatter
@@ -85,10 +141,10 @@ class ExpressionGenerator:
 
         self.ir = ir
         self.backend = backend
-        self.scope = {}
-        self._ufl_names = set()
-        self.symbol_counters = collections.defaultdict(int)
-        self.shared_symbols = {}
+        self.scope: Dict[Any, CNode] = {}
+        self._ufl_names: Set[Any] = set()
+        self.symbol_counters: DefaultDict[Any, int] = collections.defaultdict(int)
+        self.shared_symbols: Dict[Any, Any] = {}
         self.quadrature_rule = list(self.ir.integrand.keys())[0]
 
     def generate(self):
@@ -97,6 +153,8 @@ class ExpressionGenerator:
         parts = []
 
         parts += self.generate_element_tables()
+        # Generate the tables of geometry data that are needed
+        parts += self.generate_geometry_tables()
         parts += self.generate_piecewise_partition()
 
         all_preparts = []
@@ -112,6 +170,31 @@ class ExpressionGenerator:
 
         return L.StatementList(parts)
 
+    def generate_geometry_tables(self):
+        """Generate static tables of geometry data."""
+        L = self.backend.language
+
+        # Currently we only support circumradius
+        ufl_geometry = {
+            ufl.geometry.ReferenceCellVolume: "reference_cell_volume"
+        }
+        cells = {t: set() for t in ufl_geometry.keys()}
+
+        for integrand in self.ir.integrand.values():
+            for attr in integrand["factorization"].nodes.values():
+                mt = attr.get("mt")
+                if mt is not None:
+                    t = type(mt.terminal)
+                    if t in ufl_geometry:
+                        cells[t].add(mt.terminal.ufl_domain().ufl_cell().cellname())
+
+        parts = []
+        for i, cell_list in cells.items():
+            for c in cell_list:
+                parts.append(geometry.write_table(L, ufl_geometry[i], c))
+
+        return parts
+
     def generate_element_tables(self):
         """Generate tables of FE basis evaluated at specified points."""
         L = self.backend.language
@@ -122,10 +205,12 @@ class ExpressionGenerator:
         padlen = self.ir.params["padlen"]
         table_names = sorted(tables)
 
+        scalar_type = self.backend.access.parameters["scalar_type"]
+
         for name in table_names:
             table = tables[name]
             decl = L.ArrayDecl(
-                "static const ufc_scalar_t", name, table.shape, table, padlen=padlen)
+                f"static const {scalar_type}", name, table.shape, table, padlen=padlen)
             parts += [decl]
 
         # Add leading comment if there are any tables
@@ -354,6 +439,7 @@ class ExpressionGenerator:
         L = self.backend.language
 
         definitions = []
+        pre_definitions = dict()
         intermediates = []
 
         use_symbol_array = True
@@ -373,7 +459,10 @@ class ExpressionGenerator:
 
                 # Backend specific modified terminal translation
                 vaccess = self.backend.access.get(mt.terminal, mt, tabledata, 0)
-                vdef = self.backend.definitions.get(mt.terminal, mt, tabledata, 0, vaccess)
+
+                predef, vdef = self.backend.definitions.get(mt.terminal, mt, tabledata, 0, vaccess)
+                if predef:
+                    pre_definitions[str(predef[0].symbol.name)] = predef
 
                 # Store definitions of terminals in list
                 assert isinstance(vdef, list)
@@ -416,8 +505,9 @@ class ExpressionGenerator:
                         vaccess = symbol[j]
                         intermediates.append(L.Assign(vaccess, vexpr))
                     else:
+                        scalar_type = self.backend.access.parameters["scalar_type"]
                         vaccess = L.Symbol("%s_%d" % (symbol.name, j))
-                        intermediates.append(L.VariableDecl("const ufc_scalar_t", vaccess, vexpr))
+                        intermediates.append(L.VariableDecl(f"const {scalar_type}", vaccess, vexpr))
 
             # Store access node for future reference
             self.scope[v] = vaccess
@@ -425,10 +515,16 @@ class ExpressionGenerator:
         # Join terminal computation, array of intermediate expressions,
         # and intermediate computations
         parts = []
+
+        for _, definition in pre_definitions.items():
+            parts += definition
+
         if definitions:
             parts += definitions
+
         if intermediates:
             if use_symbol_array:
-                parts += [L.ArrayDecl("ufc_scalar_t", symbol, len(intermediates))]
+                scalar_type = self.backend.access.parameters["scalar_type"]
+                parts += [L.ArrayDecl(scalar_type, symbol, len(intermediates))]
             parts += intermediates
         return parts
