@@ -6,16 +6,18 @@
 
 import collections
 import logging
-from typing import Tuple, List
+from typing import List, Tuple
 
 import ufl
 from ffcx.codegeneration import geometry
-from ffcx.codegeneration import integrals_template as ufc_integrals
+from ffcx.codegeneration import integrals_template as ufcx_integrals
 from ffcx.codegeneration.backend import FFCXBackend
 from ffcx.codegeneration.C.format_lines import format_indented_lines
+from ffcx.codegeneration.C.cnodes import CNode, BinOp
 from ffcx.ir.elementtables import piecewise_ttypes
-from ffcx.ir.representationutils import QuadratureRule
 from ffcx.ir.integral import block_data_t
+from ffcx.ir.representationutils import QuadratureRule
+from ffcx.naming import cdtype_to_numpy
 
 logger = logging.getLogger("ffcx")
 
@@ -27,13 +29,9 @@ def generator(ir, parameters):
 
     """Generate code for an integral."""
     factory_name = ir.name
-    integral_type = ir.integral_type
 
     # Format declaration
-    if integral_type == "custom":
-        declaration = ufc_integrals.custom_declaration.format(factory_name=factory_name)
-    else:
-        declaration = ufc_integrals.declaration.format(factory_name=factory_name)
+    declaration = ufcx_integrals.declaration.format(factory_name=factory_name)
 
     # Create FFCx C backend
     backend = FFCXBackend(ir, parameters)
@@ -73,12 +71,15 @@ def generator(ir, parameters):
     if parameters["tabulate_tensor_void"]:
         code["tabulate_tensor"] = ""
 
-    implementation = ufc_integrals.factory.format(
+    implementation = ufcx_integrals.factory.format(
         factory_name=factory_name,
         enabled_coefficients=code["enabled_coefficients"],
         enabled_coefficients_init=code["enabled_coefficients_init"],
         tabulate_tensor=code["tabulate_tensor"],
-        needs_facet_permutations=1 if ir.needs_facet_permutations else 0)
+        needs_facet_permutations="true" if ir.needs_facet_permutations else "false",
+        scalar_type=parameters["scalar_type"],
+        np_scalar_type=cdtype_to_numpy(parameters["scalar_type"]),
+        coordinate_element=L.AddressOf(L.Symbol(ir.coordinate_element)))
 
     return declaration, implementation
 
@@ -175,9 +176,10 @@ class IntegralGenerator(object):
 
         alignment = self.ir.params['assume_aligned']
         if alignment != -1:
-            parts += [L.VerbatimStatement(f"A = (ufc_scalar_t*)__builtin_assume_aligned(A, {alignment});"),
-                      L.VerbatimStatement(f"w = (const ufc_scalar_t*)__builtin_assume_aligned(w, {alignment});"),
-                      L.VerbatimStatement(f"c = (const ufc_scalar_t*)__builtin_assume_aligned(c, {alignment});"),
+            scalar_type = self.backend.access.parameters["scalar_type"]
+            parts += [L.VerbatimStatement(f"A = ({scalar_type}*)__builtin_assume_aligned(A, {alignment});"),
+                      L.VerbatimStatement(f"w = (const {scalar_type}*)__builtin_assume_aligned(w, {alignment});"),
+                      L.VerbatimStatement(f"c = (const {scalar_type}*)__builtin_assume_aligned(c, {alignment});"),
                       L.VerbatimStatement(
                           f"coordinate_dofs = (const double*)__builtin_assume_aligned(coordinate_dofs, {alignment});")]
 
@@ -197,15 +199,23 @@ class IntegralGenerator(object):
         all_preparts = []
         all_quadparts = []
 
+        # Pre-definitions are collected across all quadrature loops
+        # to improve re-use and avoid name clashes
+        all_predefinitions = dict()
+
         for rule in self.ir.integrand.keys():
             # Generate code to compute piecewise constant scalar factors
             all_preparts += self.generate_piecewise_partition(rule)
 
             # Generate code to integrate reusable blocks of final
             # element tensor
-            preparts, quadparts = self.generate_quadrature_loop(rule)
+            pre_definitions, preparts, quadparts = self.generate_quadrature_loop(rule)
             all_preparts += preparts
             all_quadparts += quadparts
+            all_predefinitions.update(pre_definitions)
+
+        parts += L.commented_code_list(self.fuse_loops(all_predefinitions),
+                                       "Pre-definitions of modified terminals to enable unit-stride access")
 
         # Collect parts before, during, and after quadrature loops
         parts += all_preparts
@@ -317,9 +327,9 @@ class IntegralGenerator(object):
     def generate_quadrature_loop(self, quadrature_rule: QuadratureRule):
         """Generate quadrature loop with for this quadrature_rule."""
         L = self.backend.language
-
         # Generate varying partition
-        body = self.generate_varying_partition(quadrature_rule)
+        pre_definitions, body = self.generate_varying_partition(quadrature_rule)
+
         body = L.commented_code_list(
             body, f"Quadrature loop body setup for quadrature rule {quadrature_rule.id()}")
 
@@ -339,7 +349,7 @@ class IntegralGenerator(object):
             iq = self.backend.symbols.quadrature_loop_index()
             quadparts = [L.ForRange(iq, 0, num_points, body=body)]
 
-        return preparts, quadparts
+        return pre_definitions, preparts, quadparts
 
     def generate_piecewise_partition(self, quadrature_rule):
         L = self.backend.language
@@ -348,7 +358,8 @@ class IntegralGenerator(object):
         F = self.ir.integrand[quadrature_rule]["factorization"]
 
         arraysymbol = L.Symbol(f"sp_{quadrature_rule.id()}")
-        parts = self.generate_partition(arraysymbol, F, "piecewise", None)
+        pre_definitions, parts = self.generate_partition(arraysymbol, F, "piecewise", None)
+        assert len(pre_definitions) == 0, "Quadrature independent code should have not pre-definitions"
         parts = L.commented_code_list(
             parts, f"Quadrature loop independent computations for quadrature rule {quadrature_rule.id()}")
 
@@ -361,15 +372,17 @@ class IntegralGenerator(object):
         F = self.ir.integrand[quadrature_rule]["factorization"]
 
         arraysymbol = L.Symbol(f"sv_{quadrature_rule.id()}")
-        parts = self.generate_partition(arraysymbol, F, "varying", quadrature_rule)
+        pre_definitions, parts = self.generate_partition(arraysymbol, F, "varying", quadrature_rule)
         parts = L.commented_code_list(
             parts, f"Varying computations for quadrature rule {quadrature_rule.id()}")
-        return parts
+
+        return pre_definitions, parts
 
     def generate_partition(self, symbol, F, mode, quadrature_rule):
         L = self.backend.language
 
         definitions = dict()
+        pre_definitions = dict()
         intermediates = []
 
         use_symbol_array = True
@@ -393,7 +406,10 @@ class IntegralGenerator(object):
 
                     # Backend specific modified terminal translation
                     vaccess = self.backend.access.get(mt.terminal, mt, tabledata, quadrature_rule)
-                    vdef = self.backend.definitions.get(mt.terminal, mt, tabledata, quadrature_rule, vaccess)
+                    predef, vdef = self.backend.definitions.get(mt.terminal, mt, tabledata, quadrature_rule, vaccess)
+                    if predef:
+                        access = predef[0].symbol.name
+                        pre_definitions[str(access)] = predef
 
                     # Store definitions of terminals in list
                     assert isinstance(vdef, list)
@@ -438,8 +454,9 @@ class IntegralGenerator(object):
                             vaccess = symbol[j]
                             intermediates.append(L.Assign(vaccess, vexpr))
                         else:
+                            scalar_type = self.backend.access.parameters["scalar_type"]
                             vaccess = L.Symbol("%s_%d" % (symbol.name, j))
-                            intermediates.append(L.VariableDecl("const ufc_scalar_t", vaccess, vexpr))
+                            intermediates.append(L.VariableDecl(f"const {scalar_type}", vaccess, vexpr))
 
                 # Store access node for future reference
                 self.set_var(quadrature_rule, v, vaccess)
@@ -447,15 +464,15 @@ class IntegralGenerator(object):
         # Join terminal computation, array of intermediate expressions,
         # and intermediate computations
         parts = []
-        if definitions:
-            for vaccess, definition in definitions.items():
-                parts += definition
+        parts += self.fuse_loops(definitions)
+
         if intermediates:
             if use_symbol_array:
                 padlen = self.ir.params["padlen"]
-                parts += [L.ArrayDecl("ufc_scalar_t", symbol, len(intermediates), padlen=padlen)]
+                parts += [L.ArrayDecl(self.backend.access.parameters["scalar_type"],
+                                      symbol, len(intermediates), padlen=padlen)]
             parts += intermediates
-        return parts
+        return pre_definitions, parts
 
     def generate_dofblock_partition(self, quadrature_rule: QuadratureRule):
         block_contributions = self.ir.integrand[quadrature_rule]["block_contributions"]
@@ -526,8 +543,8 @@ class IntegralGenerator(object):
         L = self.backend.language
 
         # The parts to return
-        preparts = []
-        quadparts = []
+        preparts: List[CNode] = []
+        quadparts: List[CNode] = []
 
         # RHS expressiong grouped by LHS "dofmap"
         rhs_expressions = collections.defaultdict(list)
@@ -579,7 +596,8 @@ class IntegralGenerator(object):
                 key = (quadrature_rule, factor_index, blockdata.all_factors_piecewise)
                 fw, defined = self.get_temp_symbol("fw", key)
                 if not defined:
-                    quadparts.append(L.VariableDecl("const ufc_scalar_t", fw, fw_rhs))
+                    scalar_type = self.backend.access.parameters["scalar_type"]
+                    quadparts.append(L.VariableDecl(f"const {scalar_type}", fw, fw_rhs))
 
             assert not blockdata.transposed, "Not handled yet"
             A_shape = self.ir.tensor_shape
@@ -606,9 +624,9 @@ class IntegralGenerator(object):
         # List of statements to keep in the inner loop
         keep = collections.defaultdict(list)
         # List of temporary array declarations
-        pre_loop = []
+        pre_loop: List[CNode] = []
         # List of loop invariant expressions to hoist
-        hoist = []
+        hoist: List[BinOp] = []
 
         for indices in rhs_expressions:
             hoist_rhs = collections.defaultdict(list)
@@ -645,23 +663,55 @@ class IntegralGenerator(object):
                         keep[indices].append(L.float_product([statement, lhs]))
                     else:
                         t = self.new_temp_symbol("t")
-                        pre_loop.append(L.ArrayDecl("ufc_scalar_t", t, blockdims[0]))
+                        scalar_type = self.backend.access.parameters["scalar_type"]
+                        pre_loop.append(L.ArrayDecl(scalar_type, t, blockdims[0]))
                         keep[indices].append(L.float_product([statement, t[B_indices[0]]]))
                         hoist.append(L.Assign(t[B_indices[i - 1]], sum))
             else:
                 keep[indices] = rhs_expressions[indices]
 
-        hoist = L.ForRange(B_indices[0], 0, blockdims[0], body=[hoist]) if hoist else []
+        hoist_code: List[CNode] = [L.ForRange(B_indices[0], 0, blockdims[0], body=hoist)] if hoist else []
 
-        body = []
+        body: List[CNode] = []
 
         for indices in keep:
             sum = L.Sum(keep[indices])
             body.append(L.AssignAdd(A[indices], sum))
 
         for i in reversed(range(block_rank)):
-            body = L.ForRange(B_indices[i], 0, blockdims[i], body=body)
+            body = [L.ForRange(B_indices[i], 0, blockdims[i], body=body)]
 
-        quadparts += [pre_loop, hoist, body]
+        quadparts += pre_loop
+        quadparts += hoist_code
+        quadparts += body
 
         return preparts, quadparts
+
+    def fuse_loops(self, definitions):
+        """
+        Merge a sequence of loops with the same iteration space into a single loop.
+
+        Loop fusion improves data locality, cache reuse and decreases the loop control overhead.
+        NOTE: Loop fusion might increase the pressure on register allocation.
+        Ideally, we should define a cost function to determine how many loops should fuse at a time.
+        """
+        L = self.backend.language
+
+        loops = collections.defaultdict(list)
+        pre_loop = []
+        for access, definition in definitions.items():
+            for d in definition:
+                if isinstance(d, L.ForRange):
+                    loops[(d.index, d.begin, d.end)] += [d.body]
+                else:
+                    pre_loop += [d]
+        fused = []
+
+        for info, body in loops.items():
+            index, begin, end = info
+            fused += [L.ForRange(index, begin, end, body)]
+
+        code = []
+        code += pre_loop
+        code += fused
+        return code
