@@ -1,40 +1,97 @@
+# Copyright (C) 2021 Matthew W. Scroggs and Chris Richardson
+#
+# This file is part of FFCx.(https://www.fenicsproject.org)
+#
+# SPDX-License-Identifier:    LGPL-3.0-or-later
+"""Finite element interface."""
+
+from __future__ import annotations
+
+import typing
+
+if typing.TYPE_CHECKING:
+    import ufl.finiteelement.FiniteElementBase
+
+import warnings
+from abc import ABC, abstractmethod
+
+import basix
 import numpy
 import ufl
-import basix
-import warnings
+import basix.ufl_wrapper
+import functools
 
 
-def create_element(ufl_element):
-    """Create an element from a UFL element."""
+@functools.lru_cache()
+def create_basix_element(family_type, cell_type, degree, variant_info, discontinuous):
+    """Create a basix element."""
+    return basix.create_element(family_type, cell_type, degree, *variant_info, discontinuous)
+
+
+def create_element(element: ufl.finiteelement.FiniteElementBase) -> BaseElement:
+    """Create an FFCx element from a UFL element.
+
+    Args:
+        element: A UFL finite element
+
+    Returns:
+        A FFCx finite element
+    """
     # TODO: EnrichedElement
-    # TODO: Short/alternative names for elements
-    # TODO: Allow different args for different parts of mixed element
 
-    if isinstance(ufl_element, ufl.VectorElement):
-        return BlockedElement(create_element(ufl_element.sub_elements()[0]),
-                              ufl_element.num_sub_elements())
-    if isinstance(ufl_element, ufl.TensorElement):
-        return BlockedElement(create_element(ufl_element.sub_elements()[0]),
-                              ufl_element.num_sub_elements(), None)  # TODO: block shape
+    if isinstance(element, basix.ufl_wrapper.BasixElement):
+        return BasixElement(element.basix_element)
 
-    if isinstance(ufl_element, ufl.MixedElement):
-        return MixedElement([create_element(e) for e in ufl_element.sub_elements()])
+    if isinstance(element, ufl.VectorElement):
+        return BlockedElement(create_element(element.sub_elements()[0]),
+                              element.num_sub_elements())
+    if isinstance(element, ufl.TensorElement):
+        return BlockedElement(create_element(element.sub_elements()[0]),
+                              element.num_sub_elements(), None)  # TODO: block shape
 
-    if ufl_element.family() == "Quadrature":
-        return QuadratureElement(ufl_element)
+    if isinstance(element, ufl.MixedElement):
+        return MixedElement([create_element(e) for e in element.sub_elements()])
+
+    if element.family() == "Quadrature":
+        return QuadratureElement(element)
+
+    family_name = element.family()
+    discontinuous = False
+    if family_name.startswith("Discontinuous "):
+        family_name = family_name[14:]
+        discontinuous = True
+    if family_name == "DP":
+        family_name = "P"
+        discontinuous = True
+    if family_name == "DQ":
+        family_name = "Q"
+        discontinuous = True
+    if family_name == "DPC":
+        discontinuous = True
+
+    family_type = basix.finite_element.string_to_family(family_name, element.cell().cellname())
+    cell_type = basix.cell.string_to_type(element.cell().cellname())
 
     variant_info = []
+    if family_type == basix.ElementFamily.P and element.variant() == "equispaced":
+        # This is used for elements defining cells
+        variant_info = [basix.LagrangeVariant.equispaced]
+    else:
+        if element.variant() is not None:
+            raise ValueError("UFL variants are not supported by FFCx. Please wrap a Basix element directly.")
 
-    if ufl_element.family() in ["Lagrange", "Q"]:
-        if ufl_element.variant() is None:
-            variant_info.append(basix.LatticeType.equispaced)
-        else:
-            variant_info.append(basix.lattice.string_to_type(ufl_element.variant()))
+        EF = basix.ElementFamily
+        if family_type == EF.P:
+            variant_info = [basix.LagrangeVariant.gll_warped]
+        elif family_type in [EF.RT, EF.N1E]:
+            variant_info = [basix.LagrangeVariant.legendre]
+        elif family_type in [EF.serendipity, EF.BDM, EF.N2E]:
+            variant_info = [basix.LagrangeVariant.legendre, basix.DPCVariant.legendre]
+        elif family_type == EF.DPC:
+            variant_info = [basix.DPCVariant.diagonal_gll]
 
-    family_type = basix.finite_element.string_to_family(ufl_element.family(), ufl_element.cell().cellname())
-    cell_type = basix.cell.string_to_type(ufl_element.cell().cellname())
-
-    return BasixElement(family_type, cell_type, ufl_element.degree(), variant_info)
+    return BasixElement(create_basix_element(
+        family_type, cell_type, element.degree(), tuple(variant_info), discontinuous))
 
 
 def basix_index(*args):
@@ -47,7 +104,8 @@ def create_quadrature(cellname, degree, rule):
     if cellname == "vertex":
         return [[]], [1]
 
-    quadrature = basix.make_quadrature(rule, basix.cell.string_to_type(cellname), degree)
+    quadrature = basix.make_quadrature(
+        basix.quadrature.string_to_type(rule), basix.cell.string_to_type(cellname), degree)
 
     # The quadrature degree from UFL can be very high for some
     # integrals.  Print warning if number of quadrature points
@@ -70,251 +128,260 @@ def map_facet_points(points, facet, cellname):
     """Map points from a reference facet to a physical facet."""
     geom = basix.geometry(basix.cell.string_to_type(cellname))
     facet_vertices = [geom[i] for i in basix.topology(basix.cell.string_to_type(cellname))[-2][facet]]
-
     return [facet_vertices[0] + sum((i - facet_vertices[0]) * j for i, j in zip(facet_vertices[1:], p))
             for p in points]
 
 
-class BaseElement:
+class BaseElement(ABC):
     """An abstract element class."""
 
-    def tabulate(self, nderivs, points):
+    @abstractmethod
+    def tabulate(self, nderivs: int, points: numpy.ndarray):
         """Tabulate the basis functions of the element.
 
-        Parameters
-        ----------
-        nderivs : int
-            The number of derivatives to tabulate.
-        points : np.array
-            The points to tabulate at
+        Args:
+            nderivs: Number of derivatives to tabulate.
+            points: Points to tabulate at
+
+        Returns:
+            Tabulated basis functions
         """
-        raise NotImplementedError
+        pass
 
-    def get_component_element(self, flat_component):
-        """Get an element that represents a component of the element, and the offset and stride of the component.
+    @abstractmethod
+    def get_component_element(self, flat_component: int) -> tuple[BaseElement, int, int]:
+        """Get element that represents a component of the element, and the offset and stride of the component.
 
-        For example, for a MixedElement, this will return the sub-element that represents the given component,
-        the offset of that sub-element, and a stride of 1. For a BlockedElement, this will return the sub-element,
-        an offset equal to the component number, and a stride equal to the block size. For vector-valued element
-        (eg H(curl) and H(div) elements), this returns a ComponentElement (and as offset of 0 and a stride of 1).
-        When tabulate is called on the ComponentElement, only the part of the table for the given component is
-        returned.
+        For example, for a MixedElement, this will return the
+        sub-element that represents the given component, the offset of
+        that sub-element, and a stride of 1. For a BlockedElement, this
+        will return the sub-element, an offset equal to the component
+        number, and a stride equal to the block size. For vector-valued
+        element (eg H(curl) and H(div) elements), this returns a
+        ComponentElement (and as offset of 0 and a stride of 1). When
+        tabulate is called on the ComponentElement, only the part of the
+        table for the given component is returned.
 
-        Parameters
-        ----------
-        flat_component : int
-            The component
+        Args:
+            flat_component: The component
 
-        Returns
-        -------
-        BaseElement
-            The component element
-        int
-            The offset of the component
-        int
-            The stride of the component
+        Returns:
+            component element, offset of the component, stride of the component
         """
-        raise NotImplementedError
+        pass
 
     @property
     def element_type(self):
-        """Get the element type."""
+        """Element type."""
         raise NotImplementedError
 
     @property
-    def dim(self):
-        """Get the number of DOFs the element has."""
-        raise NotImplementedError
+    @abstractmethod
+    def dim(self) -> int:
+        """Number of DOFs the element has."""
+        pass
 
     @property
-    def value_size(self):
-        """Get the value size of the element."""
-        raise NotImplementedError
+    @abstractmethod
+    def value_size(self) -> int:
+        """Value size of the element.
+
+        Equal to ``numpy.prod(value_shape)``.
+
+        """
+        pass
 
     @property
-    def value_shape(self):
-        """Get the value shape of the element."""
-        raise NotImplementedError
+    @abstractmethod
+    def value_shape(self) -> typing.Tuple[int, ...]:
+        """Value shape of the element basis function.
+
+        Note:
+            For scalar elements, ``(1,)`` is returned. This is different
+            from Basix where the value shape for scalar elements is
+            ``(,)``.
+
+        """
+        pass
 
     @property
+    @abstractmethod
     def num_entity_dofs(self):
-        """Get the number of DOFs associated with each entity."""
-        raise NotImplementedError
+        """Number of DOFs associated with each entity."""
+        pass
 
     @property
+    @abstractmethod
     def entity_dofs(self):
-        """Get the DOF numbers associated with each entity."""
-        raise NotImplementedError
+        """DOF numbers associated with each entity."""
+        pass
 
     @property
+    @abstractmethod
     def num_entity_closure_dofs(self):
-        """Get the number of DOFs associated with the closure of each entity."""
-        raise NotImplementedError
+        """Number of DOFs associated with the closure of each entity."""
+        pass
 
     @property
+    @abstractmethod
     def entity_closure_dofs(self):
-        """Get the DOF numbers associated with the closure of each entity."""
-        raise NotImplementedError
+        """DOF numbers associated with the closure of each entity."""
+        pass
 
     @property
+    @abstractmethod
     def num_global_support_dofs(self):
-        """Get the number of globally supported DOFs."""
-        raise NotImplementedError
-
-    @property
-    def family_name(self):
-        """Get the family name of the element."""
-        raise NotImplementedError
+        pass
 
     @property
+    @abstractmethod
     def reference_topology(self):
-        """Get the topology of the reference element."""
-        raise NotImplementedError
+        """Topology of the reference element."""
+        pass
 
     @property
+    @abstractmethod
     def reference_geometry(self):
-        """Get the geometry of the reference element."""
-        raise NotImplementedError
+        """Geometry of the reference element."""
+        pass
 
     @property
-    def lattice_type(self):
-        """Get the lattice type used to initialise the element."""
-        raise NotImplementedError
+    @abstractmethod
+    def family_name(self) -> str:
+        """Family name of the element."""
+        pass
 
     @property
+    @abstractmethod
     def element_family(self):
-        """Get the Basix element family used to initialise the element."""
-        raise NotImplementedError
+        """Basix element family used to initialise the element."""
+        pass
 
     @property
+    @abstractmethod
+    def lagrange_variant(self):
+        """Basix Lagrange variant used to initialise the element."""
+        pass
+
+    @property
+    @abstractmethod
+    def dpc_variant(self):
+        """Basix DPC variant used to initialise the element."""
+        pass
+
+    @property
+    @abstractmethod
     def cell_type(self):
-        """Get the Basix cell type used to initialise the element."""
-        raise NotImplementedError
+        """Basix cell type used to initialise the element."""
+        pass
+
+    @property
+    @abstractmethod
+    def discontinuous(self) -> bool:
+        """True if the discontinuous version of the element is used."""
+        pass
+
+    @property
+    def is_custom_element(self) -> bool:
+        """True if the element is a custom Basix element."""
+        return False
 
 
 class BasixElement(BaseElement):
     """An element defined by Basix."""
 
-    def __init__(self, family_type, cell_type, degree, variant_info):
-        self.element = basix.create_element(family_type, cell_type, degree, *variant_info)
-        self._family = family_type
-        self._cell = cell_type
-        self._variant_info = variant_info
+    def __init__(self, element):
+        self.element = element
 
     def tabulate(self, nderivs, points):
-        """Tabulate the basis functions of the element.
-
-        Parameters
-        ----------
-        nderivs : int
-            The number of derivatives to tabulate.
-        points : np.array
-            The points to tabulate at
-        """
-        return self.element.tabulate(nderivs, points)
+        tab = self.element.tabulate(nderivs, points)
+        return tab.transpose((0, 1, 3, 2)).reshape((tab.shape[0], tab.shape[1], -1))
 
     def get_component_element(self, flat_component):
-        """Get an element that represents a component of the element, and the offset and stride of the component.
-
-        For vector-valued elements (eg H(curl) and H(div) elements), this returns a ComponentElement (and an
-        offset of 0 and a stride of 1). When tabulate is called on the ComponentElement, only the part of the
-        table for the given component is returned.
-
-        Parameters
-        ----------
-        flat_component : int
-            The component
-
-        Returns
-        -------
-        BaseElement
-            The component element
-        int
-            The offset of the component
-        int
-            The stride of the component
-        """
         assert flat_component < self.value_size
         return ComponentElement(self, flat_component), 0, 1
 
     @property
-    def element_type(self):
-        """Get the element type."""
-        return "ufc_basix_element"
+    def element_type(self) -> str:
+        """Element type."""
+        if self.is_custom_element:
+            return "ufcx_basix_custom_element"
+        else:
+            return "ufcx_basix_element"
 
     @property
     def dim(self):
-        """Get the number of DOFs the element has."""
         return self.element.dim
 
     @property
     def value_size(self):
-        """Get the value size of the element."""
         return self.element.value_size
 
     @property
     def value_shape(self):
         """Get the value shape of the element."""
-        return self.element.value_shape
+        if len(self.element.value_shape) == 0:
+            return (1,)
+        else:
+            return self.element.value_shape
 
     @property
     def num_entity_dofs(self):
-        """Get the number of DOFs associated with each entity."""
         return self.element.num_entity_dofs
 
     @property
     def entity_dofs(self):
-        """Get the DOF numbers associated with each entity."""
         return self.element.entity_dofs
 
     @property
     def num_entity_closure_dofs(self):
-        """Get the number of DOFs associated with the closure of each entity."""
         return self.element.num_entity_closure_dofs
 
     @property
     def entity_closure_dofs(self):
-        """Get the DOF numbers associated with the closure of each entity."""
         return self.element.entity_closure_dofs
 
     @property
     def num_global_support_dofs(self):
-        """Get the number of globally supported DOFs."""
         # TODO
         return 0
 
     @property
     def family_name(self):
-        """Get the family name of the element."""
         return self.element.family.name
 
     @property
     def reference_topology(self):
-        """Get the topology of the reference element."""
         return basix.topology(self.element.cell_type)
 
     @property
     def reference_geometry(self):
-        """Get the geometry of the reference element."""
         return basix.geometry(self.element.cell_type)
 
     @property
-    def lattice_type(self):
-        """Get the lattice type used to initialise the element."""
-        for a in self._variant_info:
-            if isinstance(a, basix.LatticeType):
-                return a
-        return None
+    def element_family(self):
+        return self.element.family
 
     @property
-    def element_family(self):
-        """Get the Basix element family used to initialise the element."""
-        return self._family
+    def lagrange_variant(self):
+        return self.element.lagrange_variant
+
+    @property
+    def dpc_variant(self):
+        return self.element.dpc_variant
 
     @property
     def cell_type(self):
-        """Get the Basix cell type used to initialise the element."""
-        return self._cell
+        return self.element.cell_type
+
+    @property
+    def discontinuous(self):
+        return self.element.discontinuous
+
+    @property
+    def is_custom_element(self) -> bool:
+        """True if the element is a custom Basix element."""
+        return self.element.family == basix.ElementFamily.custom
 
 
 class ComponentElement(BaseElement):
@@ -325,15 +392,6 @@ class ComponentElement(BaseElement):
         self.component = component
 
     def tabulate(self, nderivs, points):
-        """Tabulate the basis functions of the element.
-
-        Parameters
-        ----------
-        nderivs : int
-            The number of derivatives to tabulate.
-        points : np.array
-            The points to tabulate at
-        """
         tables = self.element.tabulate(nderivs, points)
         output = []
         for tbl in tables:
@@ -342,7 +400,8 @@ class ComponentElement(BaseElement):
             if len(self.element.value_shape) == 1:
                 output.append(tbl[:, self.component, :])
             elif len(self.element.value_shape) == 2:
-                # TODO: Something different may need doing here if tensor is symmetric
+                # TODO: Something different may need doing here if
+                # tensor is symmetric
                 vs0 = self.element.value_shape[0]
                 output.append(tbl[:, self.component // vs0, self.component % vs0, :])
             else:
@@ -350,41 +409,74 @@ class ComponentElement(BaseElement):
         return output
 
     def get_component_element(self, flat_component):
-        """Get an element that represents a component of the element, and the offset and stride of the component.
-
-        Parameters
-        ----------
-        flat_component : int
-            The component
-
-        Returns
-        -------
-        BaseElement
-            The component element
-        int
-            The offset of the component
-        int
-            The stride of the component
-        """
         if flat_component == 0:
             return self, 0, 1
         raise NotImplementedError
 
     @property
-    def lattice_type(self):
-        """Get the lattice type used to initialise the element."""
-        return self.element.lattice_type
+    def dim(self):
+        raise NotImplementedError
+
+    @property
+    def value_size(self):
+        raise NotImplementedError
+
+    @property
+    def value_shape(self):
+        raise NotImplementedError
+
+    @property
+    def num_entity_dofs(self):
+        raise NotImplementedError
+
+    @property
+    def entity_dofs(self):
+        raise NotImplementedError
+
+    @property
+    def num_entity_closure_dofs(self):
+        raise NotImplementedError
+
+    @property
+    def entity_closure_dofs(self):
+        raise NotImplementedError
+
+    @property
+    def num_global_support_dofs(self):
+        raise NotImplementedError
+
+    @property
+    def family_name(self) -> str:
+        raise NotImplementedError
+
+    @property
+    def reference_topology(self):
+        raise NotImplementedError
+
+    @property
+    def reference_geometry(self):
+        raise NotImplementedError
 
     @property
     def element_family(self):
-        """Get the Basix element family used to initialise the element."""
         return self.element.element_family
 
+    @property
+    def lagrange_variant(self):
+        return self.element.lagrange_variant
+
+    @property
+    def dpc_variant(self):
+        return self.element.dpc_variant
+
     @property
     def cell_type(self):
-        """Get the Basix cell type used to initialise the element."""
         return self.element.cell_type
 
+    @property
+    def discontinuous(self):
+        return self.element.discontinuous
+
 
 class MixedElement(BaseElement):
     """A mixed element that combines two or more elements."""
@@ -394,15 +486,6 @@ class MixedElement(BaseElement):
         self.sub_elements = sub_elements
 
     def tabulate(self, nderivs, points):
-        """Tabulate the basis functions of the element.
-
-        Parameters
-        ----------
-        nderivs : int
-            The number of derivatives to tabulate.
-        points : np.array
-            The points to tabulate at
-        """
         tables = []
         results = [e.tabulate(nderivs, points) for e in self.sub_elements]
         for deriv_tables in zip(*results):
@@ -416,32 +499,14 @@ class MixedElement(BaseElement):
         return tables
 
     def get_component_element(self, flat_component):
-        """Get an element that represents a component of the element, and the offset and stride of the component.
-
-        For a MixedElement, this will return the sub-element that represents the given component,
-        the offset of that sub-element, and a stride of 1.
-
-        Parameters
-        ----------
-        flat_component : int
-            The component
-
-        Returns
-        -------
-        BaseElement
-            The component element
-        int
-            The offset of the component
-        int
-            The stride of the component
-        """
         sub_dims = [0] + [e.dim for e in self.sub_elements]
         sub_cmps = [0] + [e.value_size for e in self.sub_elements]
 
         irange = numpy.cumsum(sub_dims)
         crange = numpy.cumsum(sub_cmps)
 
-        # Find index of sub element which corresponds to the current flat component
+        # Find index of sub element which corresponds to the current
+        # flat component
         component_element_index = numpy.where(
             crange <= flat_component)[0].shape[0] - 1
 
@@ -452,35 +517,30 @@ class MixedElement(BaseElement):
         return e, irange[component_element_index] + offset, stride
 
     @property
-    def element_type(self):
+    def element_type(self) -> str:
         """Get the element type."""
-        return "ufc_mixed_element"
+        return "ufcx_mixed_element"
 
     @property
     def dim(self):
-        """Get the number of DOFs the element has."""
         return sum(e.dim for e in self.sub_elements)
 
     @property
     def value_size(self):
-        """Get the value size of the element."""
         return sum(e.value_size for e in self.sub_elements)
 
     @property
     def value_shape(self):
-        """Get the value shape of the element."""
         return (sum(e.value_size for e in self.sub_elements), )
 
     @property
     def num_entity_dofs(self):
-        """Get the number of DOFs associated with each entity."""
         data = [e.num_entity_dofs for e in self.sub_elements]
         return [[sum(d[tdim][entity_n] for d in data) for entity_n, _ in enumerate(entities)]
                 for tdim, entities in enumerate(data[0])]
 
     @property
     def entity_dofs(self):
-        """Get the DOF numbers associated with each entity."""
         dofs = [[[] for i in entities] for entities in self.sub_elements[0].entity_dofs]
         start_dof = 0
         for e in self.sub_elements:
@@ -492,14 +552,12 @@ class MixedElement(BaseElement):
 
     @property
     def num_entity_closure_dofs(self):
-        """Get the number of DOFs associated with the closure of each entity."""
         data = [e.num_entity_closure_dofs for e in self.sub_elements]
         return [[sum(d[tdim][entity_n] for d in data) for entity_n, _ in enumerate(entities)]
                 for tdim, entities in enumerate(data[0])]
 
     @property
     def entity_closure_dofs(self):
-        """Get the DOF numbers associated with the closure of each entity."""
         dofs = [[[] for i in entities] for entities in self.sub_elements[0].entity_closure_dofs]
         start_dof = 0
         for e in self.sub_elements:
@@ -511,45 +569,51 @@ class MixedElement(BaseElement):
 
     @property
     def num_global_support_dofs(self):
-        """Get the number of globally supported DOFs."""
         return sum(e.num_global_support_dofs for e in self.sub_elements)
 
     @property
     def family_name(self):
-        """Get the family name of the element."""
         return "mixed element"
 
     @property
     def reference_topology(self):
-        """Get the topology of the reference element."""
         return self.sub_elements[0].reference_topology
 
     @property
     def reference_geometry(self):
-        """Get the geometry of the reference element."""
         return self.sub_elements[0].reference_geometry
 
     @property
-    def lattice_type(self):
-        """Get the lattice type used to initialise the element."""
+    def lagrange_variant(self):
+        return None
+
+    @property
+    def dpc_variant(self):
         return None
 
     @property
     def element_family(self):
-        """Get the Basix element family used to initialise the element."""
         return None
 
     @property
     def cell_type(self):
-        """Get the Basix cell type used to initialise the element."""
         return None
 
+    @property
+    def discontinuous(self):
+        return False
+
 
 class BlockedElement(BaseElement):
     """An element with a block size that contains multiple copies of a sub element."""
 
     def __init__(self, sub_element, block_size, block_shape=None):
         assert block_size > 0
+        if sub_element.value_size != 1:
+            raise ValueError("Blocked elements (VectorElement and TensorElement) of "
+                             "non-scalar elements are not supported. Try using MixedElement "
+                             "instead.")
+
         self.sub_element = sub_element
         self.block_size = block_size
         if block_shape is None:
@@ -558,18 +622,8 @@ class BlockedElement(BaseElement):
             self.block_shape = block_shape
 
     def tabulate(self, nderivs, points):
-        """Tabulate the basis functions of the element.
-
-        Parameters
-        ----------
-        nderivs : int
-            The number of derivatives to tabulate.
-        points : np.array
-            The points to tabulate at
-        """
         assert len(self.block_shape) == 1  # TODO: block shape
         assert self.value_size == self.block_size  # TODO: remove this assumption
-
         output = []
         for table in self.sub_element.tabulate(nderivs, points):
             new_table = numpy.zeros((table.shape[0], table.shape[1] * self.block_size**2))
@@ -580,106 +634,83 @@ class BlockedElement(BaseElement):
         return output
 
     def get_component_element(self, flat_component):
-        """Get an element that represents a component of the element, and the offset and stride of the component.
-
-        For a BlockedElement, this will return the sub-element, an offset equal to the component number, and a
-        stride equal to the block size.
-
-        Parameters
-        ----------
-        flat_component : int
-            The component
-
-        Returns
-        -------
-        BaseElement
-            The component element
-        int
-            The offset of the component
-        int
-            The stride of the component
-        """
         return self.sub_element, flat_component, self.block_size
 
     @property
     def element_type(self):
-        """Get the element type."""
-        return "ufc_blocked_element"
+        """Element type."""
+        return self.sub_element.element_type
 
     @property
     def dim(self):
-        """Get the number of DOFs the element has."""
         return self.sub_element.dim * self.block_size
 
     @property
     def value_size(self):
-        """Get the value size of the element."""
         return self.block_size * self.sub_element.value_size
 
     @property
     def value_shape(self):
-        """Get the value shape of the element."""
         return (self.value_size, )
 
     @property
     def num_entity_dofs(self):
-        """Get the number of DOFs associated with each entity."""
         return [[j * self.block_size for j in i] for i in self.sub_element.num_entity_dofs]
 
     @property
     def entity_dofs(self):
-        """Get the DOF numbers associated with each entity."""
-        # TODO: should this return this, or should it take blocks into account?
+        # TODO: should this return this, or should it take blocks into
+        # account?
         return [[[k * self.block_size + b for k in j for b in range(self.block_size)]
                  for j in i] for i in self.sub_element.entity_dofs]
 
     @property
     def num_entity_closure_dofs(self):
-        """Get the number of DOFs associated with the closure of each entity."""
         return [[j * self.block_size for j in i] for i in self.sub_element.num_entity_closure_dofs]
 
     @property
     def entity_closure_dofs(self):
-        """Get the DOF numbers associated with the closure of each entity."""
-        # TODO: should this return this, or should it take blocks into account?
+        # TODO: should this return this, or should it take blocks into
+        # account?
         return [[[k * self.block_size + b for k in j for b in range(self.block_size)]
                  for j in i] for i in self.sub_element.entity_closure_dofs]
 
     @property
     def num_global_support_dofs(self):
-        """Get the number of globally supported DOFs."""
         return self.sub_element.num_global_support_dofs * self.block_size
 
     @property
     def family_name(self):
-        """Get the family name of the element."""
         return self.sub_element.family_name
 
     @property
     def reference_topology(self):
-        """Get the topology of the reference element."""
         return self.sub_element.reference_topology
 
     @property
     def reference_geometry(self):
-        """Get the geometry of the reference element."""
         return self.sub_element.reference_geometry
 
     @property
-    def lattice_type(self):
-        """Get the lattice type used to initialise the element."""
-        return self.sub_element.lattice_type
+    def lagrange_variant(self):
+        return self.sub_element.lagrange_variant
+
+    @property
+    def dpc_variant(self):
+        return self.sub_element.dpc_variant
 
     @property
     def element_family(self):
-        """Get the Basix element family used to initialise the element."""
         return self.sub_element.element_family
 
     @property
     def cell_type(self):
-        """Get the Basix cell type used to initialise the element."""
         return self.sub_element.cell_type
 
+    @property
+    def discontinuous(self):
+        return self.sub_element.discontinuous
+
 
 class QuadratureElement(BaseElement):
     """A quadrature element."""
@@ -690,15 +721,6 @@ class QuadratureElement(BaseElement):
         self._ufl_element = ufl_element
 
     def tabulate(self, nderivs, points):
-        """Tabulate the basis functions of the element.
-
-        Parameters
-        ----------
-        nderivs : int
-            The number of derivatives to tabulate.
-        points : np.array
-            The points to tabulate at
-        """
         if nderivs > 0:
             raise ValueError("Cannot take derivatives of Quadrature element.")
 
@@ -708,47 +730,27 @@ class QuadratureElement(BaseElement):
         return tables
 
     def get_component_element(self, flat_component):
-        """Get an element that represents a component of the element, and the offset and stride of the component.
-
-        Parameters
-        ----------
-        flat_component : int
-            The component
-
-        Returns
-        -------
-        BaseElement
-            The component element
-        int
-            The offset of the component
-        int
-            The stride of the component
-        """
         return self, 0, 1
 
     @property
-    def element_type(self):
-        """Get the element type."""
-        return "ufc_quadrature_element"
+    def element_type(self) -> str:
+        """Element type."""
+        return "ufcx_quadrature_element"
 
     @property
     def dim(self):
-        """Get the number of DOFs the element has."""
         return self._points.shape[0]
 
     @property
     def value_size(self):
-        """Get the value size of the element."""
         return 1
 
     @property
     def value_shape(self):
-        """Get the value shape of the element."""
-        return [1]
+        return (1,)
 
     @property
     def num_entity_dofs(self):
-        """Get the number of DOFs associated with each entity."""
         dofs = []
         tdim = self._ufl_element.cell().topological_dimension()
 
@@ -766,7 +768,6 @@ class QuadratureElement(BaseElement):
 
     @property
     def entity_dofs(self):
-        """Get the DOF numbers associated with each entity."""
         start_dof = 0
         entity_dofs = []
         for i in self.num_entity_dofs:
@@ -779,35 +780,44 @@ class QuadratureElement(BaseElement):
 
     @property
     def num_entity_closure_dofs(self):
-        """Get the number of DOFs associated with the closure of each entity."""
         return self.num_entity_dofs
 
     @property
     def entity_closure_dofs(self):
-        """Get the DOF numbers associated with the closure of each entity."""
         return self.entity_dofs
 
     @property
     def num_global_support_dofs(self):
-        """Get the number of globally supported DOFs."""
         return 0
 
+    @property
+    def reference_topology(self):
+        raise NotImplementedError
+
+    @property
+    def reference_geometry(self):
+        raise NotImplementedError
+
     @property
     def family_name(self):
-        """Get the family name of the element."""
         return self._ufl_element.family()
 
     @property
-    def lattice_type(self):
-        """Get the lattice type used to initialise the element."""
+    def lagrange_variant(self):
+        return None
+
+    @property
+    def dpc_variant(self):
         return None
 
     @property
     def element_family(self):
-        """Get the Basix element family used to initialise the element."""
         return None
 
     @property
-    def cell_type(self):
-        """Get the Basix cell type used to initialise the element."""
+    def cell_type(self) -> None:
         return None
+
+    @property
+    def discontinuous(self):
+        return False
