@@ -9,10 +9,9 @@ import collections
 import logging
 
 import numpy
-
 import ufl
 import ufl.utils.derivativetuples
-from ffcx.element_interface import create_element, basix_index
+from ffcx.element_interface import basix_index, create_element
 from ffcx.ir.representationutils import (create_quadrature_points_and_weights,
                                          integral_type_to_entity_dim,
                                          map_integral_points)
@@ -27,7 +26,7 @@ piecewise_ttypes = ("piecewise", "fixed", "ones", "zeros")
 uniform_ttypes = ("fixed", "ones", "zeros", "uniform")
 
 unique_table_reference_t = collections.namedtuple(
-    "unique_table_reference",
+    "unique_table_reference_t",
     ["name", "values", "offset", "block_size", "ttype",
      "is_piecewise", "is_uniform", "is_permuted"])
 
@@ -97,13 +96,10 @@ def get_ffcx_table_values(points, cell, integral_type, ufl_element, avg, entityt
     entity_dim = integral_type_to_entity_dim(integral_type, tdim)
     num_entities = ufl.cell.num_cell_entities[cell.cellname()][entity_dim]
 
-    numpy.set_printoptions(suppress=True, precision=2)
     basix_element = create_element(ufl_element)
 
     # Extract arrays for the right scalar component
     component_tables = []
-    sh = tuple(basix_element.value_shape)
-    assert len(sh) > 0
     component_element, offset, stride = basix_element.get_component_element(flat_component)
 
     for entity in range(num_entities):
@@ -183,7 +179,7 @@ def get_modified_terminal_element(mt):
         elif ld and not mt.reference_value:
             raise RuntimeError(
                 "Local derivatives of global values not defined.")
-        element = mt.terminal.ufl_element()
+        element = mt.terminal.ufl_function_space().ufl_element()
         fc = mt.flat_component
     elif isinstance(mt.terminal, ufl.classes.SpatialCoordinate):
         if mt.reference_value:
@@ -258,10 +254,9 @@ def permute_quadrature_quadrilateral(points, reflections=0, rotations=0):
     return output
 
 
-def build_optimized_tables(
-    quadrature_rule, cell, integral_type, entitytype, modified_terminals, existing_tables,
-    rtol=default_rtol, atol=default_atol
-):
+def build_optimized_tables(quadrature_rule, cell, integral_type, entitytype,
+                           modified_terminals, existing_tables,
+                           rtol=default_rtol, atol=default_atol):
     """Build the element tables needed for a list of modified terminals.
 
     Input:
@@ -286,10 +281,10 @@ def build_optimized_tables(
     unique_elements = ufl.algorithms.sort_elements(
         ufl.algorithms.analysis.extract_sub_elements(all_elements))
     element_numbers = {element: i for i, element in enumerate(unique_elements)}
-
-    tables = existing_tables
     mt_tables = {}
 
+    _existing_tables = existing_tables.copy()
+
     for mt in modified_terminals:
         res = analysis.get(mt)
         if not res:
@@ -309,7 +304,8 @@ def build_optimized_tables(
         # the dofmap offset may differ due to restriction.
 
         tdim = cell.topological_dimension()
-        if entitytype == "facet":
+
+        if integral_type == "interior_facet":
             if tdim == 1:
                 t = get_ffcx_table_values(quadrature_rule.points, cell,
                                           integral_type, element, avg, entitytype,
@@ -353,6 +349,7 @@ def build_optimized_tables(
         # Clean up table
         tbl = clamp_table_small_numbers(t['array'], rtol=rtol, atol=atol)
         tabletype = analyse_table_type(tbl)
+
         if tabletype in piecewise_ttypes:
             # Reduce table to dimension 1 along num_points axis in generated code
             tbl = tbl[:, :, :1, :]
@@ -365,19 +362,16 @@ def build_optimized_tables(
             tbl = tbl[:1, :, :, :]
 
         # Check for existing identical table
-        xname_found = False
-        for xname in tables:
-            if equal_tables(tbl, tables[xname]):
-                xname_found = True
+        new_table = True
+        for table_name in _existing_tables:
+            if equal_tables(tbl, _existing_tables[table_name]):
+                name = table_name
+                tbl = _existing_tables[name]
+                new_table = False
                 break
 
-        if xname_found:
-            name = xname
-            # Retrieve existing table
-            tbl = tables[name]
-        else:
-            # Store new table
-            tables[name] = tbl
+        if new_table:
+            _existing_tables[name] = tbl
 
         cell_offset = 0
         basix_element = create_element(element)
