@@ -4,31 +4,15 @@
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 
-import cffi
 import numpy as np
 import pytest
 
 import ffcx.codegeneration.jit
 import ufl
+from ffcx.naming import cdtype_to_numpy
 
 
-def float_to_type(name):
-    """Map a string name to C and NumPy types"""
-    if name == "double":
-        return "double", np.float64
-    elif name == "double complex":
-        return "double _Complex", np.complex128
-    elif name == "float":
-        return "float", np.float32
-    elif name == "float complex":
-        return "float _Complex", np.complex64
-    elif name == "long double":
-        return "long double", np.longdouble
-    else:
-        raise RuntimeError("Unknown C type for: {}".format(name))
-
-
-@pytest.mark.parametrize("mode", ["double", "float", "long double", "double complex", "float complex"])
+@pytest.mark.parametrize("mode", ["double", "float", "long double", "double _Complex", "float _Complex"])
 def test_additive_facet_integral(mode, compile_args):
     cell = ufl.triangle
     element = ufl.FiniteElement("Lagrange", cell, 1)
@@ -41,7 +25,7 @@ def test_additive_facet_integral(mode, compile_args):
     for f, compiled_f in zip(forms, compiled_forms):
         assert compiled_f.rank == len(f.arguments())
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form0 = compiled_forms[0]
 
     assert form0.num_integrals(module.lib.exterior_facet) == 1
@@ -50,7 +34,7 @@ def test_additive_facet_integral(mode, compile_args):
 
     default_integral = form0.integrals(module.lib.exterior_facet)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     A = np.zeros((3, 3), dtype=np_type)
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
@@ -61,20 +45,21 @@ def test_additive_facet_integral(mode, compile_args):
                        np.sqrt(3.0), -1.0, 0.0,
                        -np.sqrt(3.0), -1.0, 0.0], dtype=np.float64)
 
+    kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
+
     for i in range(3):
         facets[0] = i
-        default_integral.tabulate_tensor(
-            ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-            ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-            ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-            ffi.cast('double *', coords.ctypes.data),
-            ffi.cast('int *', facets.ctypes.data),
-            ffi.cast('uint8_t *', perm.ctypes.data))
+        kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+               ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+               ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+               ffi.cast('double *', coords.ctypes.data),
+               ffi.cast('int *', facets.ctypes.data),
+               ffi.cast('uint8_t *', perm.ctypes.data))
 
         assert np.isclose(A.sum(), np.sqrt(12) * (i + 1))
 
 
-@pytest.mark.parametrize("mode", ["double", "float", "long double", "double complex", "float complex"])
+@pytest.mark.parametrize("mode", ["double", "float", "long double", "double _Complex", "float _Complex"])
 def test_additive_cell_integral(mode, compile_args):
     cell = ufl.triangle
     element = ufl.FiniteElement("Lagrange", cell, 1)
@@ -87,7 +72,7 @@ def test_additive_cell_integral(mode, compile_args):
     for f, compiled_f in zip(forms, compiled_forms):
         assert compiled_f.rank == len(f.arguments())
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form0 = compiled_forms[0]
 
     assert form0.num_integrals(module.lib.cell) == 1
@@ -96,7 +81,7 @@ def test_additive_cell_integral(mode, compile_args):
 
     default_integral = form0.integrals(0)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     A = np.zeros((3, 3), dtype=np_type)
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
@@ -105,20 +90,19 @@ def test_additive_cell_integral(mode, compile_args):
                        np.sqrt(3.0), -1.0, 0.0,
                        -np.sqrt(3.0), -1.0, 0.0], dtype=np.float64)
 
-    default_integral.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+    kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
+
+    kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     A0 = np.array(A)
 
     for i in range(3):
-
-        default_integral.tabulate_tensor(
-            ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-            ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-            ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+        kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+               ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+               ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+               ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
         assert np.all(np.isclose(A, (i + 2) * A0))
