@@ -4,34 +4,19 @@
 #
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 
-import cffi
 import numpy as np
 import pytest
 
 import ffcx.codegeneration.jit
+from ffcx.naming import cdtype_to_numpy
 import ufl
 import sympy
-
-
-def float_to_type(name):
-    """Map a string name to C and NumPy types"""
-    if name == "double":
-        return "double", np.float64
-    elif name == "double complex":
-        return "double _Complex", np.complex128
-    elif name == "float":
-        return "float", np.float32
-    elif name == "float complex":
-        return "float _Complex", np.complex64
-    elif name == "long double":
-        return "long double", np.longdouble
-    else:
-        raise RuntimeError("Unknown C type for: {}".format(name))
+from sympy.abc import x, y, z
 
 
 @pytest.mark.parametrize("mode,expected_result", [
     ("double", np.array([[1.0, -0.5, -0.5], [-0.5, 0.5, 0.0], [-0.5, 0.0, 0.5]], dtype=np.float64)),
-    ("double complex",
+    ("double _Complex",
      np.array(
          [[1.0 + 0j, -0.5 + 0j, -0.5 + 0j], [-0.5 + 0j, 0.5 + 0j, 0.0 + 0j],
           [-0.5 + 0j, 0.0 + 0j, 0.5 + 0j]],
@@ -51,7 +36,7 @@ def test_laplace_bilinear_form_2d(mode, expected_result, compile_args):
     for f, compiled_f in zip(forms, compiled_forms):
         assert compiled_f.rank == len(f.arguments())
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form0 = compiled_forms[0]
 
     assert form0.num_integrals(module.lib.cell) == 1
@@ -60,7 +45,7 @@ def test_laplace_bilinear_form_2d(mode, expected_result, compile_args):
 
     default_integral = form0.integrals(module.lib.cell)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     A = np.zeros((3, 3), dtype=np_type)
     w = np.array([], dtype=np_type)
 
@@ -70,11 +55,13 @@ def test_laplace_bilinear_form_2d(mode, expected_result, compile_args):
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
-    default_integral.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
+
+    kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, np.trace(kappa_value) * expected_result)
 
@@ -95,12 +82,12 @@ def test_laplace_bilinear_form_2d(mode, expected_result, compile_args):
          [[1.0 / 12.0, 1.0 / 24.0, 1.0 / 24.0], [1.0 / 24.0, 1.0 / 12.0, 1.0 / 24.0],
           [1.0 / 24.0, 1.0 / 24.0, 1.0 / 12.0]],
          dtype=np.float64)),
-    ("double complex",
+    ("double _Complex",
      np.array(
          [[1.0 / 12.0, 1.0 / 24.0, 1.0 / 24.0], [1.0 / 24.0, 1.0 / 12.0, 1.0 / 24.0],
           [1.0 / 24.0, 1.0 / 24.0, 1.0 / 12.0]],
          dtype=np.complex128)),
-    ("float complex",
+    ("float _Complex",
      np.array(
          [[1.0 / 12.0, 1.0 / 24.0, 1.0 / 24.0], [1.0 / 24.0, 1.0 / 12.0, 1.0 / 24.0],
           [1.0 / 24.0, 1.0 / 24.0, 1.0 / 12.0]],
@@ -122,27 +109,28 @@ def test_mass_bilinear_form_2d(mode, expected_result, compile_args):
     form0 = compiled_forms[0].integrals(module.lib.cell)[0]
     form1 = compiled_forms[1].integrals(module.lib.cell)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     A = np.zeros((3, 3), dtype=np_type)
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
-    form0.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel0 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form0, f"tabulate_tensor_{np_type}"))
+    kernel0(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     b = np.zeros(3, dtype=np_type)
-    form1.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), b.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+    kernel1 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form1, f"tabulate_tensor_{np_type}"))
+    kernel1(ffi.cast('{type} *'.format(type=mode), b.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, expected_result)
     assert np.allclose(b, 1.0 / 6.0)
@@ -151,7 +139,7 @@ def test_mass_bilinear_form_2d(mode, expected_result, compile_args):
 @pytest.mark.parametrize("mode,expected_result", [
     ("double", np.array([[1.0, -0.5, -0.5], [-0.5, 0.5, 0.0], [-0.5, 0.0, 0.5]], dtype=np.float64)
      - (1.0 / 24.0) * np.array([[2, 1, 1], [1, 2, 1], [1, 1, 2]], dtype=np.float64)),
-    ("double complex",
+    ("double _Complex",
      np.array([[1.0, -0.5, -0.5], [-0.5, 0.5, 0.0], [-0.5, 0.0, 0.5]], dtype=np.complex128)
      - (1.0j / 24.0) * np.array([[2, 1, 1], [1, 2, 1], [1, 1, 2]], dtype=np.complex128)),
 ])
@@ -161,7 +149,7 @@ def test_helmholtz_form_2d(mode, expected_result, compile_args):
     u, v = ufl.TrialFunction(element), ufl.TestFunction(element)
     if mode == "double":
         k = 1.0
-    elif mode == "double complex":
+    elif mode == "double _Complex":
         k = ufl.constantvalue.ComplexValue(1j)
     else:
         raise RuntimeError("Unknown mode type")
@@ -176,20 +164,21 @@ def test_helmholtz_form_2d(mode, expected_result, compile_args):
 
     form0 = compiled_forms[0].integrals(module.lib.cell)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     A = np.zeros((3, 3), dtype=np_type)
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
-    form0.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+    kernel = getattr(form0, f"tabulate_tensor_{np_type}")
+
+    kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, expected_result)
 
@@ -199,7 +188,7 @@ def test_helmholtz_form_2d(mode, expected_result, compile_args):
                          [-1 / 6, 1 / 6, 0.0, 0.0],
                          [-1 / 6, 0.0, 1 / 6, 0.0],
                          [-1 / 6, 0.0, 0.0, 1 / 6]], dtype=np.float64)),
-    ("double complex",
+    ("double _Complex",
      np.array(
          [[0.5 + 0j, -1 / 6 + 0j, -1 / 6 + 0j, -1 / 6 + 0j],
           [-1 / 6 + 0j, 1 / 6 + 0j, 0.0 + 0j, 0.0 + 0j],
@@ -221,21 +210,22 @@ def test_laplace_bilinear_form_3d(mode, expected_result, compile_args):
 
     form0 = compiled_forms[0].integrals(module.lib.cell)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     A = np.zeros((4, 4), dtype=np_type)
     w = np.array([], dtype=np_type)
     c = np.array([], dtype=np_type)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     coords = np.array([0.0, 0.0, 0.0,
                        1.0, 0.0, 0.0,
                        0.0, 1.0, 0.0,
                        0.0, 0.0, 1.0], dtype=np.float64)
-    form0.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel = getattr(form0, f"tabulate_tensor_{np_type}")
+    kernel(ffi.cast('{type} *'.format(type=mode), A.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.allclose(A, expected_result)
 
@@ -258,16 +248,17 @@ def test_form_coefficient(compile_args):
     c = np.array([], dtype=np.float64)
     perm = np.array([0], dtype=np.uint8)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
-    form0.tabulate_tensor(
-        ffi.cast('double  *', A.ctypes.data),
-        ffi.cast('double  *', w.ctypes.data),
-        ffi.cast('double  *', c.ctypes.data),
-        ffi.cast('double  *', coords.ctypes.data), ffi.NULL,
-        ffi.cast('uint8_t *', perm.ctypes.data))
+
+    kernel = getattr(form0, "tabulate_tensor_float64")
+    kernel(ffi.cast('double  *', A.ctypes.data),
+           ffi.cast('double  *', w.ctypes.data),
+           ffi.cast('double  *', c.ctypes.data),
+           ffi.cast('double  *', coords.ctypes.data), ffi.NULL,
+           ffi.cast('uint8_t *', perm.ctypes.data))
 
     A_analytic = np.array([[2, 1, 1], [1, 2, 1], [1, 1, 2]], dtype=np.float64) / 24.0
     A_diff = (A - A_analytic)
@@ -309,7 +300,7 @@ def test_subdomains(compile_args):
     assert ids[0] == 0 and ids[1] == 210
 
 
-@pytest.mark.parametrize("mode", ["double", "double complex"])
+@pytest.mark.parametrize("mode", ["double", "double _Complex"])
 def test_interior_facet_integral(mode, compile_args):
     cell = ufl.triangle
     element = ufl.FiniteElement("Lagrange", cell, 1)
@@ -322,12 +313,12 @@ def test_interior_facet_integral(mode, compile_args):
     for f, compiled_f in zip(forms, compiled_forms):
         assert compiled_f.rank == len(f.arguments())
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
 
     form0 = compiled_forms[0]
 
-    ffi = cffi.FFI()
-    c_type, np_type = float_to_type(mode)
+    ffi = module.ffi
+    np_type = cdtype_to_numpy(mode)
 
     integral0 = form0.integrals(module.lib.interior_facet)[0]
     A = np.zeros((6, 6), dtype=np_type)
@@ -344,15 +335,15 @@ def test_interior_facet_integral(mode, compile_args):
                        0.0, 1.0, 0.0,
                        1.0, 1.0, 0.0]], dtype=np.float64)
 
-    integral0.tabulate_tensor(
-        ffi.cast('{}  *'.format(c_type), A.ctypes.data),
-        ffi.cast('{}  *'.format(c_type), w.ctypes.data),
-        ffi.cast('{}  *'.format(c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.cast('int *', facets.ctypes.data),
-        ffi.cast('uint8_t *', perms.ctypes.data))
+    kernel = getattr(integral0, f"tabulate_tensor_{np_type}")
+    kernel(ffi.cast(f'{mode}  *', A.ctypes.data),
+           ffi.cast(f'{mode}  *', w.ctypes.data),
+           ffi.cast(f'{mode}  *', c.ctypes.data),
+           ffi.cast('double *', coords.ctypes.data), ffi.cast('int *', facets.ctypes.data),
+           ffi.cast('uint8_t *', perms.ctypes.data))
 
 
-@pytest.mark.parametrize("mode", ["double", "double complex"])
+@pytest.mark.parametrize("mode", ["double", "double _Complex"])
 def test_conditional(mode, compile_args):
     cell = ufl.triangle
     element = ufl.FiniteElement("Lagrange", cell, 1)
@@ -375,8 +366,8 @@ def test_conditional(mode, compile_args):
     form0 = compiled_forms[0].integrals(module.lib.cell)[0]
     form1 = compiled_forms[1].integrals(module.lib.cell)[0]
 
-    ffi = cffi.FFI()
-    c_type, np_type = float_to_type(mode)
+    ffi = module.ffi
+    np_type = cdtype_to_numpy(mode)
 
     A1 = np.zeros((3, 3), dtype=np_type)
     w1 = np.array([1.0, 1.0, 1.0], dtype=np_type)
@@ -385,11 +376,12 @@ def test_conditional(mode, compile_args):
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
-    form0.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A1.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w1.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel0 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form0, f"tabulate_tensor_{np_type}"))
+    kernel0(ffi.cast('{type} *'.format(type=mode), A1.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), w1.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     expected_result = np.array([[2, -1, -1], [-1, 1, 0], [-1, 0, 1]], dtype=np_type)
     assert np.allclose(A1, expected_result)
@@ -397,11 +389,11 @@ def test_conditional(mode, compile_args):
     A2 = np.zeros(3, dtype=np_type)
     w2 = np.array([1.0, 1.0, 1.0], dtype=np_type)
 
-    form1.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), A2.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w2.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), c.ctypes.data),
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+    kernel1 = ffi.cast(f"ufcx_tabulate_tensor_{np_type} *", getattr(form1, f"tabulate_tensor_{np_type}"))
+    kernel1(ffi.cast('{type} *'.format(type=mode), A2.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), w2.ctypes.data),
+            ffi.cast('{type} *'.format(type=mode), c.ctypes.data),
+            ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     expected_result = np.ones(3, dtype=np_type)
     assert np.allclose(A2, expected_result)
@@ -423,7 +415,7 @@ def test_custom_quadrature(compile_args):
     forms = [a]
     compiled_forms, module, code = ffcx.codegeneration.jit.compile_forms(forms, cffi_extra_compile_args=compile_args)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form = compiled_forms[0]
     default_integral = form.integrals(module.lib.cell)[0]
 
@@ -434,11 +426,12 @@ def test_custom_quadrature(compile_args):
     coords = np.array([[0.0, 0.0, 0.0],
                        [1.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
-    default_integral.tabulate_tensor(
-        ffi.cast("double *", A.ctypes.data),
-        ffi.cast("double *", w.ctypes.data),
-        ffi.cast("double *", c.ctypes.data),
-        ffi.cast("double *", coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel = getattr(default_integral, "tabulate_tensor_float64")
+    kernel(ffi.cast("double *", A.ctypes.data),
+           ffi.cast("double *", w.ctypes.data),
+           ffi.cast("double *", c.ctypes.data),
+           ffi.cast("double *", coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     # Check that A is diagonal
     assert np.count_nonzero(A - np.diag(np.diagonal(A))) == 0
@@ -454,8 +447,6 @@ def test_curl_curl(compile_args):
 
 
 def lagrange_triangle_symbolic(order, corners=[(1, 0), (2, 0), (0, 1)], fun=lambda i: i):
-    x = sympy.Symbol("x")
-    y = sympy.Symbol("y")
     from sympy import S
     poly_basis = [x**i * y**j for i in range(order + 1) for j in range(order + 1 - i)]
     # vertices
@@ -495,13 +486,13 @@ def lagrange_triangle_symbolic(order, corners=[(1, 0), (2, 0), (0, 1)], fun=lamb
 @pytest.mark.parametrize("mode", ["double"])
 @pytest.mark.parametrize("sym_fun,ufl_fun", [
     (lambda i: i, lambda i: i),
-    (lambda i: i.diff("x"), lambda i: ufl.grad(i)[0]),
-    (lambda i: i.diff("y"), lambda i: ufl.grad(i)[1])])
+    (lambda i: i.diff(x), lambda i: ufl.grad(i)[0]),
+    (lambda i: i.diff(y), lambda i: ufl.grad(i)[1])])
 @pytest.mark.parametrize("order", [1, 2, 3])
 def test_lagrange_triangle(compile_args, order, mode, sym_fun, ufl_fun):
     sym = lagrange_triangle_symbolic(order, fun=sym_fun)
     cell = ufl.triangle
-    element = ufl.FiniteElement("Lagrange", cell, order, variant="gll")
+    element = ufl.FiniteElement("Lagrange", cell, order)
     v = ufl.TestFunction(element)
 
     a = ufl_fun(v) * ufl.dx
@@ -509,13 +500,13 @@ def test_lagrange_triangle(compile_args, order, mode, sym_fun, ufl_fun):
     compiled_forms, module, code = ffcx.codegeneration.jit.compile_forms(
         forms, parameters={'scalar_type': mode}, cffi_extra_compile_args=compile_args)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form0 = compiled_forms[0]
 
     assert form0.num_integrals(module.lib.cell) == 1
     default_integral = form0.integrals(module.lib.cell)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     b = np.zeros((order + 2) * (order + 1) // 2, dtype=np_type)
     w = np.array([], dtype=np_type)
 
@@ -523,20 +514,17 @@ def test_lagrange_triangle(compile_args, order, mode, sym_fun, ufl_fun):
                        [2.0, 0.0, 0.0],
                        [0.0, 1.0, 0.0]], dtype=np.float64)
 
-    default_integral.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), b.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.NULL,
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+    kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
+    kernel(ffi.cast('{type} *'.format(type=mode), b.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+           ffi.NULL,
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     # Check that the result is the same as for sympy
     assert np.allclose(b, [float(i) for i in sym])
 
 
 def lagrange_tetrahedron_symbolic(order, corners=[(1, 0, 0), (2, 0, 0), (0, 1, 0), (0, 0, 1)], fun=lambda i: i):
-    x = sympy.Symbol("x")
-    y = sympy.Symbol("y")
-    z = sympy.Symbol("z")
     from sympy import S
     poly_basis = [
         x**i * y**j * z**k for i in range(order + 1) for j in range(order + 1 - i)
@@ -587,13 +575,13 @@ def lagrange_tetrahedron_symbolic(order, corners=[(1, 0, 0), (2, 0, 0), (0, 1, 0
 @pytest.mark.parametrize("mode", ["double"])
 @pytest.mark.parametrize("sym_fun,ufl_fun", [
     (lambda i: i, lambda i: i),
-    (lambda i: i.diff("x"), lambda i: ufl.grad(i)[0]),
-    (lambda i: i.diff("y"), lambda i: ufl.grad(i)[1])])
+    (lambda i: i.diff(x), lambda i: ufl.grad(i)[0]),
+    (lambda i: i.diff(y), lambda i: ufl.grad(i)[1])])
 @pytest.mark.parametrize("order", [1, 2, 3])
 def test_lagrange_tetrahedron(compile_args, order, mode, sym_fun, ufl_fun):
     sym = lagrange_tetrahedron_symbolic(order, fun=sym_fun)
     cell = ufl.tetrahedron
-    element = ufl.FiniteElement("Lagrange", cell, order, variant="gll")
+    element = ufl.FiniteElement("Lagrange", cell, order)
     v = ufl.TestFunction(element)
 
     a = ufl_fun(v) * ufl.dx
@@ -601,14 +589,14 @@ def test_lagrange_tetrahedron(compile_args, order, mode, sym_fun, ufl_fun):
     compiled_forms, module, code = ffcx.codegeneration.jit.compile_forms(
         forms, parameters={'scalar_type': mode}, cffi_extra_compile_args=compile_args)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form0 = compiled_forms[0]
 
     assert form0.num_integrals(module.lib.cell) == 1
 
     default_integral = form0.integrals(module.lib.cell)[0]
 
-    c_type, np_type = float_to_type(mode)
+    np_type = cdtype_to_numpy(mode)
     b = np.zeros((order + 3) * (order + 2) * (order + 1) // 6, dtype=np_type)
     w = np.array([], dtype=np_type)
 
@@ -616,11 +604,12 @@ def test_lagrange_tetrahedron(compile_args, order, mode, sym_fun, ufl_fun):
                        2.0, 0.0, 0.0,
                        0.0, 1.0, 0.0,
                        0.0, 0.0, 1.0], dtype=np.float64)
-    default_integral.tabulate_tensor(
-        ffi.cast('{type} *'.format(type=c_type), b.ctypes.data),
-        ffi.cast('{type} *'.format(type=c_type), w.ctypes.data),
-        ffi.NULL,
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel = getattr(default_integral, f"tabulate_tensor_{np_type}")
+    kernel(ffi.cast('{type} *'.format(type=mode), b.ctypes.data),
+           ffi.cast('{type} *'.format(type=mode), w.ctypes.data),
+           ffi.NULL,
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     # Check that the result is the same as for sympy
     assert np.allclose(b, [float(i) for i in sym])
@@ -636,7 +625,7 @@ def test_prism(compile_args):
     compiled_forms, module, _ = ffcx.codegeneration.jit.compile_forms(
         forms, parameters={'scalar_type': 'double'}, cffi_extra_compile_args=compile_args)
 
-    ffi = cffi.FFI()
+    ffi = module.ffi
     form0 = compiled_forms[0]
     assert form0.num_integrals(module.lib.cell) == 1
 
@@ -648,10 +637,11 @@ def test_prism(compile_args):
                        1.0, 0.0, 1.0,
                        0.0, 1.0, 1.0,
                        0.0, 0.0, 1.0], dtype=np.float64)
-    default_integral.tabulate_tensor(
-        ffi.cast('double *', b.ctypes.data),
-        ffi.NULL,
-        ffi.NULL,
-        ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
+
+    kernel = getattr(default_integral, "tabulate_tensor_float64")
+    kernel(ffi.cast('double *', b.ctypes.data),
+           ffi.NULL,
+           ffi.NULL,
+           ffi.cast('double *', coords.ctypes.data), ffi.NULL, ffi.NULL)
 
     assert np.isclose(sum(b), 0.5)
